#!/usr/bin/perl
#
# Copyright Chris Hill, Automotive Products plc, 1997
# Based on fw1-v3.pl by Feroz Khan
#
# Script to summarise the accounting logfile from Firewall-1 v3.0
#
# Input  : all exported accounting files in current directory
# Output : summarised logfile by Date/Protocol/Service/User/SrcIP/DestIP/SrcHost/DestHost 
#          accumulating No. of Sessions, Elapsed Time, Bytes Transferred
#
# Output file is in CSV format for import to PC based spreadsheet or database
#
# Called as follows:
#
# fwsumm.pl 
#
# Fields in the log file
#
# $0  = Number
# $1  = Date
# $2  = Time
# $3  = Origination
# $4  = Type
# $5  = Action (one of accept/deny)
# $6  = Alert
# $7  = Interface Name
# $8  = Interface Direction
# $9  = Protocol Type (one of TCP/UDP)
# $10 = Src Host
# $11 = Dst Host
# $12 = Type of Service (Consult /etc/services or Services table)
# $13 = Port Number for the source
# $14 = "rule"
# $15 = Elapsed time
# $16 = Packets for this session
# $17 = Number of Bytes
# $18 = Authenticated User Name
# $19 = Messages
#

print "********** fwsumm start ****************\n";

%mnths = (	"Jan" => "01",
		"Feb" => "02",
		"Mar" => "03",
		"Apr" => "04",
		"May" => "05",
		"Jun" => "06",
		"Jul" => "07",
		"Aug" => "08",
		"Sep" => "09",
		"Oct" => "10",
		"Nov" => "11",
		"Dec" => "12"	);

%myhosts = (	'aaa.bbb.cc1'		=> 'My internal network',
		'aaa.bbb.cc2'		=> 'My internal network',
  		'aaa.bbb.cc3'		=> 'My internal network',
    		'aaa.bbb.cc4'		=> 'My internal network',
    		'xxx.yyy.zzz'		=> 'My external network',
		'aaa.bbb.cc0.2'		=> 'My internal DMZ DNS',
		'aaa.bbb.cc0.3'		=> 'My Internal DMZ Mail',
		'aaa.bbb.cc0.4'		=> 'My Internal DMZ News',
		'aaa.bbb.cc0.5'		=> 'My Internal DMZ WWW',
		'aaa.bbb.cc0.6'		=> 'My Internal DMZ FTP',
		'xxx.yyy.zzz.nn0'	=> 'My Firewall !!!',
		'xxx.yyy.zzz.nn1'	=> 'My External DMZ FTP',
		'xxx.yyy.zzz.nn2'	=> 'My External DMZ WWW',
		'xxx.yyy.zzz.nn3'	=> 'My External DMZ News',
		'xxx.yyy.zzz.nn4'	=> 'My External DMZ Mail',
		'xxx.yyy.zzz.nn5'	=> 'My External DMZ DNS');
 

$dir = "/opt/SUNWfw/log/";

opendir DIR, $dir;
@export = grep /\.export$/, readdir DIR;
closedir DIR;

foreach $export (@export)
{
  print "Processing $export\n"; 
  open EXP,"$dir$export" ;
  while (<EXP>)
  {
    next if /^num/;		# skip headings line
    next if /^$/;		# skip blank line

    chomp;			# remove \n
    @fields = split(/\;/);	# parse fields

# Convert the date from D(D)mmmYY to DD/MM/YY format

    @w = split(/ */,$fields[1]);

    if ( $w[1] lt '0' or $w[1] gt '9')
    {
    $day = join("","0",$w[0]);		# day < 10
    $mnth = join("",$w[1],$w[2],$w[3]);
    $year = join("",$w[4],$w[5]);
    }
    else
    {
    $day = join("",$w[0],$w[1]);	# day >= 10
    $mnth = join("",$w[2],$w[3],$w[4]);
    $year = join("",$w[5],$w[6]);
    } 
    $date = join("/",$day,$mnths{$mnth},$year);

    $key = join(",",$date,$fields[9],$fields[12],$fields[18],$fields[10],$fields[11]);

# split the elapsed time into hour/minute/seconds

    ($hr, $min, $sec ) = split(/:/, $fields[15]);

# Accumulate Sessions, Time and Bytes by key

    $sessions{$key} += 	1;
    $time{$key}     += ((($hr * 60) + $min) * 60) + $sec ;
    $bytes{$key}    += $fields[17];

# Remember IP addresses

    $iphost{$fields[10]} = '';
    $iphost{$fields[11]} = '';
 
  }
  close EXP;
# last;
}

# Resolve the IP addresses

print "Resolving IP Addresses\n";
$apaddrs = 0;
$resaddrs = 0;
$unresaddrs = 0;

foreach $ipaddr (sort keys %iphost)
{
  next if $ipaddr =~ /^$/;	# skip blank entries

  @ip = split(/\./,$ipaddr);
  $net = join('.',$ip[0],$ip[1],$ip[2]);

  if (defined $myhosts{$ipaddr})
  {
    $iphost{$ipaddr} = $myhosts{$ipaddr};
    $apaddrs += 1;
  }
  elsif (defined $myhosts{$net})
  {
    $iphost{$ipaddr} = $myhosts{$net};
    $apaddrs += 1;
  }
  else
  {
    $iphost{$ipaddr} = gethostbyaddr(pack('C4',@ip),2);
    if ($iphost{$ipaddr} =~ /^\w/)
    {
       $resaddrs += 1;
    }
    else
    {
      $iphost{$ipaddr} = "UNKNOWN HOST";
      $unresaddrs += 1;
    }
  }

  print "IP addr $ipaddr =====> Host $iphost{$ipaddr} \n"; 
}

print "Resolved : $apaddrs My IP Address(es) : $resaddrs External IP Address(es) : $unresaddrs Unresolved Address(es)\n";

# Open summary file

@time = localtime;
$time[3] = '00'.$time[3];
$day = substr($time[3],-2);
$time[4] += 1;
$time[4] = '00'.$time[4];
$mnth = substr($time[4],-2);

$summary = join("",$dir,"fw.$time[5]$mnth$day.summary");
open SUMM, ">>$summary" or die 'unable to open summary file';

print "Generating summary output to $summary\n";
$summrecs = 0;

# Output for each key
foreach $record (sort keys %sessions )
{
  @fields = split(/,/,$record);
  print SUMM join(',',$record,$iphost{$fields[4]},$iphost{$fields[5]},$sessions{$record},$time{$record},$bytes{$record}),"\n";
# print      join(',',$record,$iphost{$fields[4]},$iphost{$fields[5]},$sessions{$record},$time{$record},$bytes{$record}),"\n";
  $summrecs +=1;
}
close SUMM;

print "Generated $summrecs records\n";

print "********** fwsumm end ****************\n";
